clear all; 

fprintf('\n \t ........... [COMPARISON PADUA POINTS], OCTOBER 2, 2007 ........... \n')

N=5:1:30;              % (HYPER)INTERPOLATION DEGREE.

ftype=12;              % CHOOSING FUNCTION TYPE: SEE FILE "fct2D.m".
                       % [ftype=1]   z=franke(x,y);
                       % [ftype=2]   z=( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
                       % [ftype=3]   z=(x+y).^20;
                       % [ftype=4]   z=exp(loc_arg); loc_arg=(x-0.5).^2+(y-0.5).^2; 
                       % [ftype=5]   z=exp(-100*loc_arg); loc_arg=(x-0.5).^2+(y-0.5).^2;
                       % [ftype=7]   z=ones(size(x));
                       % [ftype=6]   z=cos(30*(x+y));
                       % [ftype=8]   z=exp(x+y);
                       % [ftype=9]   z=1./den; den=1+16*(x.^2+y.^2);
                       % [ftype=10]  z=(x.^2+y.^2).^(3/2);
                       % [ftype=11]  z=exp(-x.^2-y.^2);
                       % [ftype=12]  z=exp(-(x.^2+y.^2).^(-1));
                       %
                       % TESTS 3, 8, 9, 10, 11, 12 ARE MODIFICATIONS OF 1D EXAMPLES BY L.N.TREFETHEN.

xrange=[-1 1];         % RANGE OF THE PROBLEM.
yrange=[-1 1];  

exact_cubature=[];    
                       % SOME EXAMPLES:
                       % [ftype=3]  exact_cubature= (2^23)/(21*22)
                       %                          = 18157.160173160173000000000000000000
                       % [ftype=8]  exact_cubature=     5.524391382167264600000000000000
                       % [ftype=9]  exact_cubature=     0.59738894727430714000
                       % [ftype=10] exact_cubature= 23/16*2^(1/2)-3/16*log(2^(1/2)-1)+(3/4)*0.33404479474209392000
                       %                            -(3/8)*(-4/25)                          
                       %                          = 2.50872313953405920000
                       % [ftype=11] exact_cubature= ( (erf(1)-erf(-1)) * (sqrt(pi)/2) )^2
                       %                          =     2.230985141404133700000000000000
                       % [ftype=12] exact_cubature=0.85335875865430844000 (NUMERICALLY COMPUTED!)
                       % IF YOU DO NOT KNOW THE RIGHT RESULT, PLEASE LEAVE IT EMPTY.

degree_high_order_rule=400;       
                       % DEGREE OF THE RULE USED BY TENSORIAL GL RULE TO COMPUTE EXACT INTEGRAL.

make_plot=1;           % [make_plot=0] NO PLOT (RELATIVE CUBATURE ERRORS). 
                       % [make_plot=1] PLOT (RELATIVE CUBATURE ERRORS). 
                       
                       

%------------------------------------------------------------------------------------------------------
% MAIN PROGRAM: IT STARTS HERE.
%------------------------------------------------------------------------------------------------------

if length(exact_cubature) < 1                                           %------------------------
        [exact_cubature,nodes,weights_TGL]=tensorial_integration(...    % HIGH ORDER TENS. RULE.
            @fct2D,degree_high_order_rule,4,ftype);                     %------------------------
end

fprintf('\n \t ------------------------------------------------------------------------ ')
fprintf('\n \t [FUNCTION]: %3.0f',ftype)
fprintf('\n \t ------------------------------------------------------------------------ ')

for index=1:length(N)
    
    n=N(index);    
    
    [I_mpx,nodes_mpx,weights_mpx]=quadPD(@fct2D, n, ...                  %-----------------
        xrange, yrange, 1, ftype);                                       % MPX CUBATURE.
    pts_mpx(index)=size(nodes_mpx,1);                                    %-----------------
    
    
    [I_pd,nodes_PD,weights_PD]=quadPD(@fct2D, n, ...                     %-----------------
        xrange, yrange, 2, ftype);                                       % PADUA CUBATURE.
    pts_PD(index)=size(nodes_PD,1);                                      %-----------------
    
    
    n_tens(index)=fix(sqrt(pts_PD(index))); 
                                                                         %-----------------
    [cub_tens_CC,nodes,weights_TCC]=tensorial_integration(@fct2D,...     % TENS. CC.
        n_tens(index),3,ftype);                                          %-----------------                                                     
    
                                                                         %-----------------
    [cub_tens_GL,nodes,weights_TGL]=tensorial_integration(@fct2D,...     % TENS. GL.
        n_tens(index),4,ftype);                                          %-----------------                            
    
    pts_tens(index)=(length(nodes))^2;
    
    err_mpx(index)=step_error(exact_cubature,I_mpx);                                                                 
    err_PD(index)=step_error(exact_cubature,I_pd);                       %-----------------
    err_CC(index)=step_error(exact_cubature,cub_tens_CC);                % RELATIVE ERRORS.
    err_GL(index)=step_error(exact_cubature,cub_tens_GL);                %-----------------
    
     fprintf('\n \t [DEG.]: %3.0f [PTS][MPX]: %5.0f [PD]: %5.0f [TN]: %5.0f ',...
       n,pts_mpx(index),pts_PD(index),pts_tens(index));
    
    fprintf('\n \t [ERR][MPX]: %1.1e [PD]: %1.1e [CC]: %1.1e [GL]: %1.1e',...
        err_mpx(index),err_PD(index),err_CC(index),err_GL(index));
    
    fprintf('\n \t ------------------------------------------------------------------------ ')
    
end

fprintf('\n \t [EXACT INTEGRAL]: %15.20f',exact_cubature);
fprintf('\n \t ------------------------------------------------------------------------ ')

%------------------------------------------------------------------------------------------------------
% FEW POINTS HIGH ORDER RULE (BY OSMOLYAN,SOLOVYAN, JCAM 188, ISSUE 2, 15 April 2006, Pages 190-204).
%------------------------------------------------------------------------------------------------------
err_qms=[]; pts_qms=[];

for N=15:2:23
    
    [w_qms,x,y]=osmolyan_solovyan_rule(N);
    f_xy=fct2D(x,y,ftype);
    I_qms=w_qms'*f_xy;
    err_qms=[err_qms; step_error(exact_cubature,I_qms)];
    n_pts_qms=length(w_qms); pts_qms=[pts_qms; n_pts_qms];
    fprintf('\n \t [OS RULES] [N]: %2.0f [PTS]: %6.0f [ERR.QMS.]: %2.2e ',N,n_pts_qms,err_qms(end));
    
end

fprintf('\n \t ------------------------------------------------------------------------ ')

%------------------------------------------------------------------------------------------------------
% PLOTS.
%------------------------------------------------------------------------------------------------------

if make_plot == 1
    
    clf
    semilogy(pts_qms,err_qms,'r-*',pts_mpx,err_mpx,'g-o', pts_PD,err_PD,'k-o',pts_tens,err_CC,'m-+',pts_tens,err_GL,'b-+');
    
    legend('OS pts.','Non tens. CC MPX pts.','Non tens. CC Padua pts.','Tens. CC pts.','Tens. GL pts.');
    xlabel('Number of points'); ylabel('Relative Cubature Error');

end
    