
function [cub,nodes,weights]=tensorial_integration(intfcn, n, cub_type, varargin)


[nodes,weights]=quadrature_rules(n,cub_type); % QUADRATURE WEIGHTS IN 
                                                    % THE INTERVAL "[-1,1]".

[x,y]=meshgrid(nodes);
f = fcnchk(intfcn);
f_xy=feval(f,x,y,varargin{:});

[s,t]=meshgrid(weights);

cub=sum(sum(s.*t.*f_xy));



%----------------------------------------------------------------------
% 1. "quadrature_rules"
%----------------------------------------------------------------------

function [nodes,weights]=quadrature_rules(n,quadrature_type)

% SEE WALDVOGEL PAPER. ADDED NODES

% Weights of the Fejer2, Clenshaw-Curtis and Fejer1 quadrature by DFTs
% n>1. Nodes: x_k = cos(k*pi/n)

N=[1:2:n-1]'; l=length(N); m=n-l; K=[0:m-1]';

switch quadrature_type
    
case 1 % FEJER 1.
v0=[2*exp(i*pi*K/n)./(1-4*K.^2); zeros(l+1,1)];
v1=v0(1:end-1)+conj(v0(end:-1:2)); 
weights=ifft(v1);
k=(1/2):(n-(1/2)); nodes=(cos(k*pi/n))'; 

case 2 % FEJER 2.
v0=[2./N./(N-2); 1/N(end); zeros(m,1)];
v2=-v0(1:end-1)-v0(end:-1:2); 
wf2=ifft(v2); weights=[wf2;0];
k=0:n; nodes=(cos(k*pi/n))'; 

case 3 % CLENSHAW CURTIS.
g0=-ones(n,1); g0(1+l)=g0(1+l)+n; g0(1+m)=g0(1+m)+n;
g=g0/(n^2-1+mod(n,2));
v0=[2./N./(N-2); 1/N(end); zeros(m,1)];
v2=-v0(1:end-1)-v0(end:-1:2);
wcc=ifft(v2+g); weights=[wcc;wcc(1,1)];
k=0:n; nodes=(cos(k*pi/n))'; 

case 4 % GAUSS LEGENDRE
beta=0.5./sqrt(1-(2*(1:n)).^(-2));
T=diag(beta,1)+diag(beta,-1);
[V,D]=eig(T);
x=diag(D); [x,index]=sort(x); x=x';
w=2*V(1,index).^2; 
nodes=x';
weights=w';

end